@@ -34,6 +34,7 @@
 #include <sys/time.h>
 #undef LIST_INIT
 #undef LIST_FOREACH
+#undef LIST_FOREACH_SAFE
 #endif
 #include <re_types.h>
 #include <re_fmt.h>
@@ -610,7 +611,13 @@ int fd_listen(struct re_fhs **fhsp, re_sock_t fd, int flags, fd_h *fh,
 
 		DEBUG_INFO("fd_listen/new: fd=%d flags=0x%02x\n", fd, flags);
 
-		++re->nfds;
+		if (++re->nfds > re->maxfds) {
+			DEBUG_WARNING("fd_listen maxfds reached %d > %d\n",
+				      re->nfds, re->maxfds);
+			--re->nfds;
+			err = EMFILE;
+			goto out;
+		}
 	}
 	else {
 		if (unlikely(fhs->fd != fd)) {
@@ -649,6 +656,7 @@ int fd_listen(struct re_fhs **fhsp, re_sock_t fd, int flags, fd_h *fh,
 		break;
 	}
 
+out:
 	if (err) {
 		mem_deref(fhs);
 		DEBUG_WARNING("fd_listen err: fd=%d flags=0x%02x (%m)\n", fd,
@@ -1647,3 +1655,23 @@ void re_thread_async_main_cancel(intptr_t id)
 
 	re_async_cancel(re->async, id);
 }
+
+
+/**
+ * Flush file descriptors handlers if re loop is not running
+ */
+void re_fhs_flush(void)
+{
+	struct re *re = re_get();
+	if (!re) {
+		DEBUG_WARNING("re_fhs_flush: re not ready\n");
+		return;
+	}
+
+	if (re_atomic_rlx(&re->polling)) {
+		DEBUG_WARNING("re_fhs_flush: re polling is running\n");
+		return;
+	}
+
+	fhsld_flush(re);
+}
