@@ -285,11 +285,7 @@ int udp_listen(struct udp_sock **usp, const struct sa *local,
 		(void)re_snprintf(serv, sizeof(serv), "%u", sa_port(local));
 	}
 	else {
-#ifdef HAVE_INET6
 		af = AF_UNSPEC;
-#else
-		af = AF_INET;
-#endif
 	}
 
 	memset(&hints, 0, sizeof(hints));
@@ -560,8 +556,6 @@ int udp_send(struct udp_sock *us, const struct sa *dst, struct mbuf *mb)
  * @param local The returned local network address
  *
  * @return 0 if success, otherwise errorcode
- *
- * @todo bug no way to specify AF
  */
 int udp_local_get(const struct udp_sock *us, struct sa *local)
 {
@@ -632,6 +626,7 @@ int udp_settos(struct udp_sock *us, uint8_t tos)
 {
 	int err = 0;
 	int v = tos;
+	struct sa sa;
 #ifdef WIN32
 	QOS_VERSION qos_version = { 1 , 0 };
 	QOS_TRAFFIC_TYPE qos_type = QOSTrafficTypeBestEffort;
@@ -664,7 +659,20 @@ int udp_settos(struct udp_sock *us, uint8_t tos)
 			return WSAGetLastError();
 	}
 #endif
-	err = udp_setsockopt(us, IPPROTO_IP, IP_TOS, &v, sizeof(v));
+	err = udp_local_get(us, &sa);
+	if (err)
+		return err;
+
+	if (sa_af(&sa) == AF_INET) {
+		err = udp_setsockopt(us, IPPROTO_IP, IP_TOS, &v, sizeof(v));
+	}
+#if defined(IPV6_TCLASS) && !defined(WIN32)
+	else if (sa_af(&sa) == AF_INET6) {
+		err = udp_setsockopt(us, IPPROTO_IPV6, IPV6_TCLASS, &v,
+				     sizeof(v));
+	}
+#endif
+
 	return err;
 }
 
@@ -735,17 +743,12 @@ void udp_error_handler_set(struct udp_sock *us, udp_error_h *eh)
  * Get the File Descriptor from a UDP Socket
  *
  * @param us  UDP Socket
- * @param af  Address Family [deprecated, ignored]
  *
  * @return File Descriptor, or RE_BAD_SOCK for errors
  */
-re_sock_t udp_sock_fd(const struct udp_sock *us, int af)
+re_sock_t udp_sock_fd(const struct udp_sock *us)
 {
-	(void)af;
-	if (!us)
-		return RE_BAD_SOCK;
-
-	return us->fd;
+	return us ? us->fd : RE_BAD_SOCK;
 }
 
 
@@ -897,11 +900,11 @@ void udp_recv_helper(struct udp_sock *us, const struct sa *src,
 	struct sa hsrc;
 	struct le *le;
 
-	if (!us || !src || !mb || !uhx)
+	if (!us || !src || !mb)
 		return;
 
 	mtx_lock(us->lock);
-	le = uhx->le.next;
+	le = uhx ? uhx->le.next : us->helpers.head;
 	mtx_unlock(us->lock);
 	while (le) {
 		struct udp_helper *uh = le->data;
@@ -989,32 +992,5 @@ void udp_flush(const struct udp_sock *us)
 void udp_recv_packet(struct udp_sock *us, const struct sa *src,
 		     struct mbuf *mb)
 {
-	struct sa hsrc;
-	struct le *le;
-
-	if (!us || !src || !mb)
-		return;
-
-	mtx_lock(us->lock);
-	le = us->helpers.head;
-	mtx_unlock(us->lock);
-	while (le) {
-		struct udp_helper *uh = le->data;
-		bool hdld;
-
-		mtx_lock(us->lock);
-		le = le->next;
-		mtx_unlock(us->lock);
-
-		if (src != &hsrc) {
-			sa_cpy(&hsrc, src);
-			src = &hsrc;
-		}
-
-		hdld = uh->recvh(&hsrc, mb, uh->arg);
-		if (hdld)
-			return;
-	}
-
-	us->rh(src, mb, us->arg);
+	udp_recv_helper(us, src, mb, NULL);
 }
